
/**
 * Google マップ インスタンス。
 */
var map;

/**
 * 住所と座標の相互変換をおこなう Geocoder インスタンス。
 */
var geo;

/**
 *  住所取得のタスクを管理するためのオブジェクト。
 */
var addressTask = {
	/**
	 * 現在、実行されているタスクの ID。
	 */
	currentId : 0,
	
	/**
	 * 住所の取得が実行中であることを示す値。
	 */
	isRunning : false
};

/**
 * Titanium への通知をおこないます。
 *
 * @param {String} func   Titanium 上のイベント ハンドラ名。
 * @param {Object} params パラメータ。
 */
function notifyTitanium( func, params ) {
	try {
		Titanium.App.fireEvent( func, params );
	} catch( e ) {
		// 通知先がない...
	}
}

/**
 * マップの中央位置が変更されたときに発生します。
 */
function onMapCenterChanged() {
	// このイベントはマップの移動中、大量に呼び出される。また、住所取得のコストが大きいので、
	// 住所更新の処理は識別子を割り当てたうえで遅延実行し、最後におこなわれたものだけ確定させる。
	//
	var id = ++addressTask.currentId;
	setTimeout( function() {
		if( addressTask.currentId != id ) { return; }

		geo.geocode( { latLng: map.getCenter(), language: "ja" }, function( results, status ) {
			if( results && results[ 0 ] ) {
				notifyTitanium( "onAddressChanged", { address: results[ 0 ].formatted_address } );
			}
		} );

	}, 200 );
}

/**
 * 指定された住所の座標へ、マップを移動します。
 *
 * @param {String} address 住所。
 */
function moveMapToAddress( address ) {
	geo.geocode( { address: address }, function( results, status ) {
		if( results && results[ 0 ] ) {
			map.setCenter( results[ 0 ].geometry.location );
			onMapCenterChanged();
		}
	});
}

/**
 * ページが読み込まれたときに呼び出されます。
 */
window.onload = function() {
	var options = {
		zoom:         16,
		center:       new google.maps.LatLng( 35.6894876, 139.6917064 ),
		mapTypeId:    google.maps.MapTypeId.ROADMAP,
		scaleControl: true
	};

	map = new google.maps.Map( document.getElementById( "map_canvas" ), options );
	geo = new google.maps.Geocoder();

	// マップ中央の更新
	google.maps.event.addListener( map, "center_changed", function() {
		setTimeout( onMapCenterChanged, 200 );
	} );

	// タイル読み込み完了
	var isInitialized = false;
	google.maps.event.addListener( map, "tilesloaded", function() {
		// 起動時に一度だけ座標を更新
		if( !isInitialized ) {
			isInitialized = true;
			onMapCenterChanged();
		}
	});
};
